#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2009 by Daniel Martin Yerga
# <dyerga@gmail.com>
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# CasualServices: Search and share your favourite places from 11870.com
# Version 0.1
#

VERSION = "0.1"

import gtk
import hildon
import gobject
import logging
import os
import thread
import osmgpsmap
import pango
import gettext
import osso

try:
    import location
except:
    print 'location is not available'

from services import once

ossoc = osso.Context("org.maemo.cservices", VERSION, False)

#Some useful long functions used in the code, we use smaller alias
fhsize = gtk.HILDON_SIZE_FINGER_HEIGHT
horbtn = hildon.BUTTON_ARRANGEMENT_HORIZONTAL
ui_normal = gtk.HILDON_UI_MODE_NORMAL
#Show a busy indication in the window menu
winprogind = hildon.hildon_gtk_window_set_progress_indicator

HOME = os.path.expanduser("~")
configdir = HOME + '/.cservices/'
logfile = configdir + 'log.txt'

if not os.path.exists(configdir):
    os.mkdir(configdir)

#detect if is running locally or not
#if it's running in /opt, it will be in the N900
import sys
runningpath = sys.path[0]

if '/opt' in runningpath:
    locally = False
else:
    locally = True

#Get the right paths of useful directories
if locally:
    imgdir = 'pixmaps/'
else:
    appdir = '/opt/cservices/'
    imgdir = appdir + 'pixmaps/'

#Define the logger used to manager the error/debug log
logger = logging.getLogger('cservices')
logging.basicConfig(filename=logfile,level=logging.ERROR, filemode='w')

DEBUG = True

if DEBUG:
    #set the main logger to DEBUG
    logger.setLevel(logging.DEBUG)

    #Create a handler for console debug
    console = logging.StreamHandler()
    console.setLevel(logging.DEBUG)
    # set a format which is simpler for console use
    formatter = logging.Formatter('%(name)-12s: %(levelname)-8s %(message)s')
    # tell the handler to use this format
    console.setFormatter(formatter)
    logging.getLogger('').addHandler(console)
    logger.debug("Starting log")


titles = []
phones = []
urls = []
localities = []
gmlpos = []
summaries = []
addresses = []
countries = []
areas = []

def file_browser(window, action, title, folder, filename, file_ext):
    """Return a filename chosen from a Hildon.FileChooserDialog

    @type   window: gtk.Widget
    @param  window: Widget to be parent of the dialog
    @type   action: gtk.FILE_CHOOSER_ACTION
    @param  action: The open or save mode for the dialog
    @type   title: string
    @param  title: Title of the dialog
    @type   folder: string
    @param  folder: Current folder for the FileChooserDialog
    @type   filename: string
    @param  filename: Filename of the file to be saved
    @type   file_ext: string
    @param  file_ext: Filename extension


    @rtype: string
    @returns: String, a complete path+filename chosen for the user

    """

    #This is a generic FileChooserDialog
    #Can be used for Open or Save actions

    m = hildon.FileSystemModel()
    file_dialog = hildon.FileChooserDialog(window, action, m)
    file_dialog.set_title(title)

    file_dialog.set_default_response(gtk.RESPONSE_CANCEL)
    file_dialog.set_current_folder(folder)
    if (action==gtk.FILE_CHOOSER_ACTION_SAVE):
        file_dialog.set_current_name(filename)

    file_dialog.show_all()
    result = file_dialog.run()

    if result == gtk.RESPONSE_OK:
        namefile = file_dialog.get_filename()

        if (action==gtk.FILE_CHOOSER_ACTION_SAVE):
            namefile, extension = os.path.splitext(namefile)
            namefile = namefile + "." + file_ext
    else:
        namefile = False
    file_dialog.destroy()

    return namefile


gtk.gdk.threads_init()

class MainWindow:
    """Class to create and use the UI of the application

    """
    def __init__(self):
        """Start the application UI.

        """
        #Define where gettext should search for translation files
        gettext.install('cservices','/opt/cservices/share/locale')
        self.program = hildon.Program()
        self.program.__init__()
        gtk.set_application_name("CasualServices")

        self.window = hildon.StackableWindow()
        self.window.set_default_size(800, 480)
        self.window.connect("destroy", gtk.main_quit)
        self.program.add_window(self.window)

        #Variable for pagination in searches, the first page is 1 obviously
        self.page = '1'

        #Create the widgets of the main window
        vbox = self.initial_screen()
        self.window.add(vbox)

        #Create the menu, and set it to the main window
        menu = self.make_menu()
        self.window.set_app_menu(menu)

        self.window.show_all()

        #Hide the buttons to change the page in the searches
        self.backbtn.hide()
        self.forwbtn.hide()

    def initial_screen(self):
        """Return a gtk.VBox with the UI widgets for the initial screen.

        @rtype: gtk.VBox
        @returns: gtk.VBox with the UI widgets for the initial screen.

        """

        vbox = gtk.VBox()

        parea = hildon.PannableArea()
        #This signal is for the gestures to change page in searches
        #It is not really gestures but horizontal movement
        parea.connect("horizontal-movement", self.horizontal_mov)

        self.searchtv = hildon.GtkTreeView(ui_normal)

        #Create an empty search model, to start w/o results
        self.search_model = self.create_search_model('')
        self.searchtv.set_model(self.search_model)

        self.searchtv.connect("row-activated", self.selected_service)

        self.add_columns_to_searchlist(self.searchtv)

        parea.add(self.searchtv)

        #Toolbar of the main window
        toolbar = self.maintoolbar()

        vbox.pack_start(parea, True, True, 0)
        vbox.pack_start(toolbar, False, False, 0)

        return vbox

    def horizontal_mov(self, parea, direction, initial_x, initial_y):
        """Search in next or previous page with gestures

        @type   parea: hildon.PannableArea
        @param  parea: hildon.PannableArea to receive the signal
        @type   direction: number
        @param  direction: direction of the movement
        @type   initial_x: number
        @param  initial_x: the x coordinate of the point where the user clicked
        @type   initial_y: number
        @param  initial_y: the y coordinate of the point where the user clicked

        """
        #Direction 2 -> left movement in the pannable area
        #Direction 3 -> right movement in the pannable area
        if direction == 2:
            self.search_prev_page(parea)
        elif direction == 3:
            self.search_next_page(parea)

    def maintoolbar(self):
        """Return a gtk.HBox with the UI widgets for the main window toolbar.

        @rtype: gtk.HBox
        @returns: gtk.HBox with the UI widgets for the main window toolbar.

        """

        hbox = gtk.HBox()

        #button = hildon.CheckButton(fhsize)
        #img = gtk.Image()
        #img.set_from_file('/usr/share/icons/hicolor/48x48/hildon/camera_gps_location_pressed.png')
        #button.set_image(img)
        #button.connect("clicked", self.remove_recipes)
        #hbox.pack_start(button, False, False, 5)

        #Text entry for searches
        self.searchentry = hildon.Entry(fhsize)
        self.searchentry.connect("activate", self.search_services)
        self.searchentry.set_placeholder("Search...")
        hbox.pack_start(self.searchentry, True, True, 0)

        #Search button
        button = hildon.Button(fhsize, horbtn)
        img = gtk.Image()
        img.set_from_file('/usr/share/icons/hicolor/48x48/hildon/general_search.png')
        button.set_image(img)
        button.connect("clicked", self.search_services)
        hbox.pack_start(button, False, False, 5)

        #Previous page button (for searches)
        self.backbtn = hildon.Button(fhsize, horbtn)
        img = gtk.Image()
        img.set_from_file('/usr/share/icons/hicolor/48x48/hildon/general_back.png')
        self.backbtn.set_image(img)
        self.backbtn.connect("clicked", self.search_prev_page)
        hbox.pack_start(self.backbtn, False, False, 5)

        #Next page button (for searches)
        self.forwbtn = hildon.Button(fhsize, horbtn)
        img = gtk.Image()
        img.set_from_file('/usr/share/icons/hicolor/48x48/hildon/general_forward.png')
        self.forwbtn.set_image(img)
        self.forwbtn.connect("clicked", self.search_next_page)
        hbox.pack_start(self.forwbtn, False, False, 5)

        return hbox

    def make_menu(self):
        """Return a hildon.AppMenu with the menu items for the main window.

        @rtype: hildon.AppMenu
        @returns: hildon.AppMenu with the menu items for the main window.

        """
        menu = hildon.AppMenu()

        button = hildon.CheckButton(fhsize)
        button.set_label(_("Activate GPS"))
        button.set_active(False)
        menu.append(button)
        button.connect("toggled", self.activate_gps)

        self.nearbtn = hildon.CheckButton(fhsize)
        self.nearbtn.set_label(_("Search near you"))
        self.nearbtn.set_active(False)
        menu.append(self.nearbtn)

        button = gtk.Button(_("Validate 11870.com"))
        button.connect("clicked", self.validate_once)
        menu.append(button)

        button = gtk.Button(_("About"))
        button.connect("clicked", About)
        menu.append(button)

        menu.show_all()
        return menu

    def validate_once(self, widget):
        """Check if exists authentication token in the configuration file.
        If it does not exist, try get one from 11870.com

        @type   widget: gtk.Widget
        @param  widget: gtk widget that emitted the signal

        """
        onceAPI = once.API()

        if onceAPI.has_authtoken():
            #Get the authentication token from the config file
            self.authtoken = onceAPI.get_authtoken()
            #print 'has authtoken: ', self.authtoken
            #Show banner to the user to note that does not need a authtoken
            self.show_info_banner(widget,
                            _("Authorization to 11870.com already completed"))

        else:
            #Authentication token is not available, then try to get one
            temptoken = onceAPI.get_temptoken()

            #Launch the web browser in a different thread
            thread.start_new_thread(onceAPI.validate_on_web, (temptoken,))

            #Show the dialog to validate the authentication
            dialog = self.validate_dialog()
            if dialog:
                authtoken = onceAPI.get_authtoken_on_web(temptoken)
                onceAPI.save_authtoken(authtoken)
                self.show_info_banner(widget,
                            _("Authorization to 11870.com successful"))
            else:
                #If it's not possible get an authtoken or if the user cancel it
                self.authenticate = False
                msg = _(("Authorization to 11870.com failed\n"
                "You will not able to use some 11870.com features"))
                self.show_info_banner(widget, msg)


    def validate_dialog(self):
        """Launch a dialog to accept the validation from 11870.com webpage

        """
        dialog = gtk.Dialog(title=_('Authorize 11870.com'), parent=self.window,
                            buttons=("OK", gtk.RESPONSE_OK))
        dialog.set_transient_for(self.window)
        label1 = gtk.Label(_('Press OK after you have given authorization.'))
        dialog.vbox.pack_start(label1, True, True, 0)

        dialog.show_all()
        result = dialog.run()
        if result == gtk.RESPONSE_OK:
            dialog.destroy()
            return True
        #else:
        #    dialog.destroy()
        #    return False

        dialog.destroy()

    def search_prev_page(self, widget):
        """Search the previous page from a previous search on 11870.com

        """
        self.page = str(int(self.page)-1)
        #Do a new search with the new page
        self.search_services(widget)

    def search_next_page(self, widget):
        """Search the next page from a previous search on 11870.com

        """
        self.page = str(int(self.page)+1)
        #Do a new search with the new page
        self.search_services(widget)

    def create_search_model(self, searchlist):
        """Return a gtk.ListStore with the results of a search on 11870.com

        @type   searchlist: list
        @param  searchlist: list of results from a search on 11870.com

        @rtype: gtk.ListStore
        @returns: Formatted gtk.ListStore with the results of a search on 11870.com

        """
        import style
        lstore = gtk.ListStore(gobject.TYPE_STRING)

        #This is to use small text in the search list
        #This text is the location of the service
        #So, it show in big text the name of the service,
        #and in small text its location
        head_font = style.get_font_desc('SystemFont')
        head_color = style.get_color('ButtonTextColor')
        head = (head_font.to_string(), head_color.to_string())
        head = '<span font_desc="%s" foreground="%s">%%s</span>' % head
        sub_font = style.get_font_desc('SmallSystemFont')
        sub_color = style.get_color('SecondaryTextColor')
        sub = (sub_font.to_string(), sub_color.to_string())
        sub = '<span font_desc="%s" foreground="%s">%%s - %%s - %%s</span>' % sub
        lstore._markup_template = '\n'.join((head, sub))

        searchlist = [titles, localities, areas, countries]

        if searchlist:
            #If there search results, add it to the gtk.liststore
            for i in range(len(searchlist[0])):
                txtformatted = lstore._markup_template % (searchlist[0][i],
                                searchlist[1][i], searchlist[2][i],
                                searchlist[3][i])
                lstore.append([txtformatted])

        return lstore

    def add_columns_to_searchlist(self, treeview):
        """Add gtk.TreeviewColumns to the gtk.TreeView for searchs

        @type   treeview: gtk.Treeview
        @param  treeview: gtk.Treeview where the columns will be added
        """
        model = treeview.get_model()

        renderer = gtk.CellRendererText()
        #Wrap the name of the service if it's bigger than 790
        #N900 screen resolution is 800 of width
        renderer.set_property('wrap-mode', pango.WRAP_WORD)
        renderer.set_property('wrap-width', 790)
        column = gtk.TreeViewColumn('Title', renderer, markup=0)
        column.set_property("expand", True)
        treeview.append_column(column)

    def selected_service(self, widget, path, column):
        """Show a new window with the information from a selected service

        @type   widget: gtk.Widget
        @param  widget: gtk.Treeview from the search that emits the signal
        @type   path: number
        @param  path: path from the selected row of the treeview
        @type   column: gtk.TreeViewColumn
        @param  column: selected column from the treeview

        """
        win = hildon.StackableWindow()
        win.set_title(_("Information"))

        vbox = gtk.VBox()

        label = gtk.Label()
        label.set_markup('<b>%s:</b> %s' % (_("Title"), titles[path[0]]))
        label.set_line_wrap(gtk.WRAP_WORD)
        vbox.pack_start(label, True, True, 0)

        label = gtk.Label()
        label.set_line_wrap(True)
        label.set_markup('<b>%s:</b> %s' % (_("Summary"), summaries[path[0]]))
        label.set_line_wrap(gtk.WRAP_WORD)
        vbox.pack_start(label, True, True, 0)

        label = gtk.Label()
        label.set_markup('<b>%s:</b> %s' % (_("Phone"), phones[path[0]]))
        label.set_line_wrap(gtk.WRAP_WORD)
        vbox.pack_start(label, True, True, 0)

        label = gtk.Label()
        label.set_markup('<b>%s:</b> %s' % (_("Address"), addresses[path[0]]))
        label.set_line_wrap(gtk.WRAP_WORD)
        vbox.pack_start(label, True, True, 0)

        label = gtk.Label()
        label.set_markup('<b>%s:</b> %s' % (_("Locality"), localities[path[0]]))
        label.set_line_wrap(gtk.WRAP_WORD)
        vbox.pack_start(label, True, True, 0)

        label = gtk.Label()
        label.set_markup('<b>%s:</b> %s' % (_("Area"), areas[path[0]]))
        label.set_line_wrap(gtk.WRAP_WORD)
        vbox.pack_start(label, True, True, 0)

        label = gtk.Label()
        label.set_markup('<b>%s:</b> %s' % (_("Country"), countries[path[0]]))
        label.set_line_wrap(gtk.WRAP_WORD)
        vbox.pack_start(label, True, True, 0)

        #label = gtk.Label()
        #label.set_markup('<b>GML Pos:</b> %s' % gmlpos[path[0]])
        #vbox.pack_start(label, True, True, 0)

        label = gtk.Label()
        label.set_markup('<b>URL:</b> %s' % urls[path[0]])
        label.set_line_wrap(gtk.WRAP_WORD)
        vbox.pack_start(label, True, True, 0)

        hbox = gtk.HBox()
        parea = hildon.PannableArea()
        parea.set_size_request(500, -1)
        parea.add_with_viewport(vbox)
        hbox.pack_start(parea, False, False, 0)

        vbox = gtk.VBox()

        #TODO: a general button istead of checkbutton??
        button = hildon.CheckButton(fhsize)
        button.connect("toggled", self.show_your_position_on_map)
        button.set_label(_("Show your position"))
        vbox.pack_start(button, False, False, 0)

        #TODO: option for choosing MAP (google, openstreetmap, etc.)
        #self.map = osmgpsmap.GpsMap(repo_uri=osmgpsmap.source_get_repo_uri(osmgpsmap.SOURCE_GOOGLE_STREET))

        self.map = osmgpsmap.GpsMap()
        lat, lon = gmlpos[path[0]].split(' ')

        pixbuf = gtk.gdk.pixbuf_new_from_file(imgdir + 'place.png')
        #FIXME: if lon and lat are not available, this fails
        self.map.add_image(float(lat), float(lon), pixbuf)
        vbox.pack_start(self.map, True, True, 0)

        #FIXME: use icons for these buttons
        hbox1 = gtk.HBox()
        button = hildon.Button(fhsize, horbtn)
        button.connect("clicked", self.zoomin_map)
        button.set_label("+")
        hbox1.pack_start(button, True, True, 0)

        button = hildon.Button(fhsize, horbtn)
        button.connect("clicked", self.center_map_on_service, lat, lon)
        button.set_label("*")
        hbox1.pack_start(button, True, True, 0)

        button = hildon.Button(fhsize, horbtn)
        button.connect("clicked", self.zoomout_map)
        button.set_label("-")
        hbox1.pack_start(button, True, True, 0)

        vbox.pack_start(hbox1, False, False, 0)

        hbox.pack_start(vbox, True, True, 0)

        win.add(hbox)
        win.show_all()
        #Centering the map in the service position
        self.map.set_mapcenter (float(lat), float(lon), 13)

    def show_your_position_on_map(self, widget):
        """Show the position of the GPS in the map

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal

        """
        if widget.get_active():
            self.activate_gps(widget)

            #FIXME: wait for a real GPS fix to show the position in the map

            lat, lon = self.gps.get_data()
            print lat, lon

            #Draw our position in the map
            self.map.draw_gps(float(lat), float(lon), 1.0)
        else:
            self.map.clear_gps()

    def zoomin_map(self, widget):
        """Zoom in the map

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal

        """
        self.map.set_zoom(self.map.props.zoom + 1)

    def zoomout_map(self, widget):
        """Zoom out the map

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal

        """
        self.map.set_zoom(self.map.props.zoom - 1)

    def center_map_on_service(self, widget, lat, lon):
        """Center the map where the service is located

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal
        @type   lat: number
        @param  lat: Geographic latitude of the service
        @type   lon: number
        @param  lon: Geographic longitude of the service

        """
        self.map.set_mapcenter (float(lat), float(lon), 13)

    def search_services(self, widget):
        """Start a new thread to search for services

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal

        """
        #If there are spaces, replace it for + to search in 11870.com
        text = self.searchentry.get_text().replace(' ', '+')
        winprogind(self.window, 1)
        #New thread to do the search, we don't wanna hang the UI
        thread.start_new(self._do_search_services, (widget, text))

    def _do_search_services(self, widget, text):
        """Search for services in 11870.com, and show the results in the UI

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal
        @type   text: string
        @param  text: text to be searched

        """

        try:
            #Clearing the previous search
            self.search_model.clear()
            del titles[:]
            del phones[:]
            del urls[:]
            del localities[:]
            del gmlpos[:]
            del summaries[:]
            del addresses[:]
            del countries[:]
            del areas[:]

            #if some data is not available, show this
            nodata = _('Not available')

            #Call the 11870.com API
            self.onceAPI = once.API()

            #if option to search "near your position" is active
            if self.nearbtn.get_active():
                #Activate the GPS to check your position
                self.activate_gps(widget)
                lat, lon = self.gps.get_data()
                #FIXME: GPS stuff
                #FIXME: raidus should be configurable
                feed = self.onceAPI.searchgeo(text, self.page, lat, lon)
            else:
                feed = self.onceAPI.search(text, self.page)

            #Add the obtained data to the different lists
            for i in feed['entries']:
                if i.has_key('title'):
                    titles.append(i['title'])
                else:
                    titles.append(nodata)

                if i.has_key('telephone'):
                    phones.append(i['telephone'])
                else:
                    phones.append(nodata)

                if i.has_key('url'):
                    urls.append(i['url'])
                else:
                    urls.append(nodata)

                if i.has_key('locality'):
                   localities.append(i['locality'])
                else:
                   localities.append(nodata)

                if i.has_key('pos'):
                    gmlpos.append(i['pos'])
                else:
                    gmlpos.append(nodata)

                if i.has_key('summary'):
                    summaries.append(i['summary'])
                else:
                    summaries.append(nodata)

                if i.has_key('useraddress'):
                   addresses.append(i['useraddress'])
                else:
                   addresses.append(nodata)

                if i.has_key('country'):
                   countries.append(i['country'])
                else:
                   countries.append(nodata)

                if i.has_key('subadministrativearea'):
                   areas.append(i['subadministrativearea'])
                else:
                   areas.append(nodata)

            searchlist = [titles, localities, areas, countries]

            #Show the search results in the UI
            self.search_model = self.create_search_model(searchlist)
            self.searchtv.set_model(self.search_model)
            winprogind(self.window, 0)

            #If there are not any results, show a banner
            if searchlist == [[], [], [], []]:
                self.show_banner_thread(self.window,
                                        _('No results for this search'))
            else:
                #Show the buttons to change the page in the search
                self.backbtn.show()
                self.forwbtn.show()
        except:
            logger.exception('Searching services')
            winprogind(self.window, 0)
            self.show_banner_thread(self.window, _('Error searching services'))

    def show_info_banner(self, widget, msg):
        """Show a information banner

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal
        @type   msg: string
        @param  msg: text to be shown in the banner

        """
        hildon.hildon_banner_show_information(widget, 'qgn_note_infoprint', msg)

    def show_banner_thread(self, widget, msg):
        """Show a information banner inside a thread

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal
        @type   msg: string
        @param  msg: text to be shown in the banner

        """
        gtk.gdk.threads_enter()
        self.show_info_banner(win, msg)
        gtk.gdk.threads_leave()

    def activate_gps(self, widget):
        """Activate N900 GPS

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal

        """
        self.gps = GPS()


#TODO: make this in the same class, easier to get data

class GPS:
    """Class to manage the functions of the N900 GPS

    """
    def __init__(self):
        """Start the N900 GPS

        """
        self.control = location.GPSDControl.get_default()
        self.device = location.GPSDevice()
        self.control.set_properties(preferred_method=location.METHOD_USER_SELECTED,
                               preferred_interval=location.INTERVAL_DEFAULT)

        self.control.connect("error-verbose", self.on_error)
        self.device.connect("changed", self.on_changed, self.control)

        #Start the GPS system
        gobject.idle_add(self.start_location, self.control)

    def on_error(self, control, error):
        """Show GPS error if something fails

        @type   control: location.GPSDControl
        @param  control: GPSDControl object that emits the signal
        @type   error: number
        @param  error: error number of the GPS system

        """
        print "location error: %d... quitting" % error

    def on_changed(self, device, control):
        """Print latitude and longitude when the GPS has a fix

        @type   device: location.GPSDevice()
        @param  device: GPSDevice object that emits the signal
        @type   control: location.GPSDControl
        @param  control: GPSDControl object that emits the signal

        """
        if not device:
            return
        if device.fix:
            if device.fix[1] & location.GPS_DEVICE_LATLONG_SET:
                print "lat = %f, long = %f" % device.fix[4:6]
                control.stop()

    def start_location(self, control):
        """Start the GPSDControl object

        @type   control: location.GPSDControl
        @param  control: GPSD

        """
        control.start()
        return False

    def get_data(self):
        """Return the current latitude and longitude of the GPS

        @rtype: numbers
        @returns: latitude, longitude

        """
        #print "online: ", self.device.online
        #print "status: ", self.device.status
        #print "fix: ", self.device.fix
        #print "sats in view: ", self.device.satellites_in_view
        #print "sats in use: ", self.device.satellites_in_use
        #print "sats: ", self.device.satellites
        #print "cellinfo: ", self.device.cell_info

        #FIXME: solo cuando hay un fix

        lat, lon = self.device.fix[4:6]
        return lat, lon


class About:
    """Class for the About dialog

    """
    def __init__(self, widget):
        """Show the About dialog UI

        """
        self.abdialog = gtk.Dialog()
        self.abdialog.set_title(_("About CasualServices"))

        notebook = gtk.Notebook()
        notebook.set_show_tabs(False)
        notebook.set_scrollable(False)
        notebook.set_show_border(False)

        # Description page #
        vbox = gtk.VBox()

        label = gtk.Label()
        label.set_markup("<b><big>CasualServices %s</big></b>" % VERSION)
        vbox.pack_start(label, True, True, 0)

        label = gtk.Label("Search and share your favourite places")
        vbox.pack_start(label, True, True, 0)

        label = gtk.Label("GNU General Public License")
        vbox.pack_start(label, True, True, 0)

        url = "http://cusl4-cservices.forja.rediris.es/"
        webbtn = gtk.LinkButton(url, "Web")
        vbox.pack_start(webbtn, True, True, 0)
        gtk.link_button_set_uri_hook(self.launch_browser)

        label = gtk.Label(_("CasualServices uses the 11870.com API"))
        vbox.pack_start(label, True, True, 0)

        url = "http://11870.com"
        webbtn = hildon.Button(fhsize, horbtn)
        webbtn.set_label("Visit")
        webbtn.connect("clicked", self.launch_browser, url)
        image = gtk.Image()
        image.set_from_file(imgdir + '11870.png')
        webbtn.set_image(image)

        hbox = gtk.HBox()
        hbox.set_homogeneous(True)
        hbox.pack_start(webbtn, False, False, 0)
        vbox.pack_start(hbox, False, False, 0)


        notebook.append_page(vbox, gtk.Label())

        # Credits page #
        vbox = gtk.VBox()
        textview = hildon.TextView()
        textview.set_cursor_visible(False)
        textview.set_wrap_mode(gtk.WRAP_WORD)
        text = ("%s Daniel Martin Yerga (dyerga@gmail.com)\n"
        "\n"
        "%s:") % (_("Written by"), _("Translations"))
        textview.get_buffer().set_text(text)

        parea = hildon.PannableArea()
        parea.add(textview)

        vbox.pack_start(parea, True, True, 0)
        notebook.append_page(vbox, gtk.Label())

        # Report page #
        vbox = gtk.VBox()

        textview = hildon.TextView()
        textview.set_cursor_visible(False)
        textview.set_wrap_mode(gtk.WRAP_WORD)
        text = _("""Please, if you find some error in the application, report it:
1) Send the log from the button above (if there's an error in the log).
2) Press the button and read how to report a bug.""")
        textview.get_buffer().set_text(text)

        parea = hildon.PannableArea()
        parea.add(textview)

        hbox = gtk.HBox()
        hbox.set_homogeneous(True)

        button = hildon.Button(fhsize, horbtn)
        button.set_title(_("Report error"))
        url = "http://cusl4-cservices.forja.rediris.es/reporting.html"
        button.connect("clicked", self.launch_browser, url)
        hbox.pack_start(button, True, True, 0)

        button = hildon.Button(fhsize, horbtn)
        button.set_title(_("Log"))
        button.connect("clicked", Log, logfile)
        hbox.pack_start(button, True, True, 0)

        vbox.pack_start(hbox, False, False, 0)
        vbox.pack_start(parea, True, True, 0)

        notebook.append_page(vbox, gtk.Label())

        # Buttons #
        self.abdialog.vbox.pack_start(notebook, True, True, 0)

        hbox = gtk.HBox()

        descbutton = hildon.GtkRadioButton(fhsize)
        descbutton.set_mode(False)
        descbutton.set_active(True)
        descbutton.set_label(_('Description'))
        descbutton.connect("toggled", self.change_tab, notebook, 0)
        hbox.pack_start(descbutton, True, True, 0)

        button = hildon.GtkRadioButton(fhsize)
        button.set_mode(False)
        button.set_active(True)
        button.set_label(_('Credits'))
        button.set_group(descbutton)
        button.connect("toggled", self.change_tab, notebook, 1)
        hbox.pack_start(button, True, True, 0)

        button = hildon.GtkRadioButton(fhsize)
        button.set_mode(False)
        button.set_label(_('Report'))
        button.set_group(descbutton)
        button.connect("clicked", self.change_tab, notebook, 2)
        hbox.pack_start(button, True, True, 0)

        self.abdialog.vbox.pack_start(hbox, False, False, 0)

        self.abdialog.show_all()
        self.abdialog.run()
        self.abdialog.destroy()

    def change_tab(self, widget, notebook, number):
        notebook.set_current_page(number)

    def launch_browser(self, widget, url):
        """Launch the webbrowser

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal
        @type   url: string
        @param  url: URL address to be opened by the browser

        """
        import dbus
        bus = dbus.SystemBus()
        proxy = bus.get_object("com.nokia.osso_browser",
                                "/com/nokia/osso_browser/request")
        iface = dbus.Interface(proxy, 'com.nokia.osso_browser')

        self.abdialog.destroy()

        iface.open_new_window(url)

class Log:
    """Class for show and manage the error log system

    """
    def __init__(self, widget, logfile):
        """Show the main dialog to manage the Log

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal
        @type   logfile: string
        @param  logfile: filename for the log file.

        """

        #Log dialog UI
        dialog = gtk.Dialog(title='Log', parent=None)

        dialog.set_size_request(600, 350)

        parea = hildon.PannableArea()
        parea.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)

        textview = hildon.TextView()
        textview.set_property("editable", False)
        textview.set_property("wrap-mode", gtk.WRAP_WORD)

        log = open(logfile, 'r')
        logtext = log.read()
        log.close()

        textview.get_buffer().set_text(logtext)
        parea.add(textview)

        dialog.vbox.pack_start(parea, True, True, 0)

        hbox = gtk.HBox()

        save_btn = hildon.Button(fhsize, horbtn)
        save_btn.set_title("Save")
        save_btn.connect('clicked', self.save, logfile, dialog)

        clear_btn = hildon.Button(fhsize, horbtn)
        clear_btn.set_title("Clear")
        clear_btn.connect('clicked', self.clear, textview, logfile)

        send_btn = hildon.Button(fhsize, horbtn)
        send_btn.set_title('Send')
        send_btn.connect('clicked', self.send, dialog, logfile)

        hbox.pack_start(save_btn, True, True, 0)
        hbox.pack_start(clear_btn, True, True, 0)
        hbox.pack_start(send_btn, True, True, 0)

        dialog.vbox.pack_start(hbox, False, False, 0)

        dialog.show_all()
        dialog.run()
        dialog.destroy()

    def clear(self, widget, textview, logfile):
        """Clear the log

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal
        @type   textview: gtk.TextView
        @param  textview: gtk.TextView that has the log written
        @type   logfile: string
        @param  logfile: filename for the log file.

        """
        textview.get_buffer().set_text('')
        f = open(logfile, 'w')
        f.close()

    def save(self, widget, logfile, dlg):
        """Save the log as a text file

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal
        @type   logfile: string
        @param  logfile: filename for the log file.
        @type   dlg: gtk.Dialog
        @param  dlg: main dialog for the Log system

        """
        import shutil
        #The user choose the filename for the log file
        filename = file_browser(dlg, gtk.FILE_CHOOSER_ACTION_SAVE,
                                _("Save log file"), HOME + '/MyDocs/.documents',
                                "cservices-log", "txt")
        if not filename:
            return

        try:
            #Copy the current log file to the path chosen for the user
            shutil.copyfile(logfile, filename)
            MainWindow.show_info_banner(widget, _('Log file saved'))
        except:
            logger.exception("Saving log file")
            MainWindow.show_info_banner(widget, _('Error saving the log file'))

    def send(self, widget, dlg, logfile):
        """Show the confirmation dialog to send the log to the server

        @type   widget: gtk.Widget
        @param  widget: gtk.Widget that emits the signal
        @type   logfile: string
        @param  logfile: filename for the log file.
        @type   dlg: gtk.Dialog
        @param  dlg: main dialog for the Log system

        """
        sendtxt = _("""You are going to send the log to the developers.
This helps the developers to track problems with the application.
It doesn't send any personal information (like passwords or similar).""")

        dialog = hildon.hildon_note_new_confirmation(dlg, sendtxt)
        dialog.set_button_texts("Send", "Cancel")
        dialog.show_all()
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            self.do_pre_send(dlg, logfile)

        dialog.destroy()

    def do_pre_send(self, dlg, logfile):
        hildon.hildon_gtk_window_set_progress_indicator(dlg, 1)
        thread.start_new_thread(self._do_send, (dlg, logfile))

    def _do_send(self, dlg, logfile):
        """Send the log to the developer server

        @type   logfile: string
        @param  logfile: filename for the log file.
        @type   dlg: gtk.Dialog
        @param  dlg: main dialog for the Log system

        """
        import pycurl, shutil, random, commands
        try:
            #Create a random name for the file to be sent
            rname = ''
            for i in random.sample('abcdefghijkl123456789', 18):
                rname += i

            #Copy the logfile to the new temporal file
            rnamepath = HOME + "/.cservices/" + rname
            shutil.copyfile(logfile, rnamepath)

            #Get different debug info, python, gtk, maemo versions, etc.
            gtkversion = "%s.%s.%s" % gtk.ver
            if os.path.exists("/etc/maemo_version"):
                mfile = open("/etc/maemo_version", 'r')
                maemoversion = mfile.read()
                mfile.close()
            else:
                maemoversion = ''

            #Check dirty memory, useful for python apps
            opsystem = ' '.join(os.uname())
            pyversion = os.sys.version
            pid = os.getpid()
            comm = ("awk '/Private_Dirty/{sum+=$2}END{print sum \"kB\"}'"
            " /proc/%s/smaps") % pid
            status, dirtymem = commands.getstatusoutput(comm)

            lfile = open(rnamepath, 'r')
            log = lfile.read()
            lfile.close()

            _version = "CasualServices %s" % VERSION

            #Formatting the text to be sent
            log = ("%s\nPython version: %s\nGtk version: %s\n"
            "Maemo version: %sOperating system: %s\n"
            "Dirty Memory: %s\nLog:\n%s") % (_version, pyversion, gtkversion,
            maemoversion, opsystem, dirtymem, log)

            lfile = open(rnamepath, 'w')
            lfile.write(log)
            lfile.close()

            #Sent the log file to the server
            url = "http://yerga.net/logs/uploader.php"
            data = [('uploadedfile', (pycurl.FORM_FILE, rnamepath)),]
            mycurl = pycurl.Curl()
            mycurl.setopt(pycurl.URL, url)
            mycurl.setopt(pycurl.HTTPPOST, data)

            mycurl.perform()
            mycurl.close()
            os.remove(rnamepath)

            gtk.gdk.threads_enter()
            MainWindow.show_info_banner(dlg, 'Log sent')
            gtk.gdk.threads_leave()
            hildon.hildon_gtk_window_set_progress_indicator(dlg, 0)
        except:
            logger.exception("Sending log file")
            gtk.gdk.threads_enter()
            MainWindow.show_info_banner(dlg, _('Error sending the log file'))
            gtk.gdk.threads_leave()
            hildon.hildon_gtk_window_set_progress_indicator(dlg, 0)


if __name__ == "__main__":
    MainWindow = MainWindow()
    gtk.gdk.threads_enter()
    gtk.main()
    gtk.gdk.threads_leave()
